import logging
import gffutils

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns

from pathlib import Path
from pansg._logging import set_logging_level


def process_genes(row, gene_lengths):
    genes = row['gene'].split(',')
    if row['gene_number'] > row['sample_number']:
        # 按材料分组
        materials = {}
        for gene in genes:
            material, gene_id = gene.split('_', 1)
            if material not in materials:
                materials[material] = []
            materials[material].append(gene_id)

        # 保留每种材料长度最接近中位数的基因
        selected_genes = []
        for material, gene_ids in materials.items():
            lengths = [gene_lengths[f'{material}_{gene_id}'] for gene_id in gene_ids]
            median_length = np.median(lengths)
            closest_gene = min(gene_ids, key=lambda gene_id: abs(gene_lengths[f'{material}_{gene_id}'] - median_length))
            selected_genes.append(f'{material}_{closest_gene}')
        row['gene'] = ','.join(selected_genes)
    return row

def gene_length_from_gff(sample_name, gff_pwd) -> dict:
    len_dic = {}
    db_path = str(Path(gff_pwd).with_suffix('.db'))

    # 读取GFF数据库
    db = gffutils.FeatureDB(db_path)

    for gene in db.features_of_type('gene'):
        transcripts = list(db.children(gene, featuretype='mRNA'))
        if not transcripts:
            continue

        # 计算每个转录本的外显子长度之和，并找到最长的转录本
        longest_transcript_length = max(
            sum(exon.end - exon.start + 1 for exon in db.children(transcript, featuretype='exon'))
            for transcript in transcripts
        )

        # 将结果存储在字典中
        len_dic[sample_name + "_" + gene.id] = longest_transcript_length

    return len_dic


def polish_pan_matrix(pan_matrix_path: str, name_list: list,gene_len_dic: dict):
    order_list = ['SG', 'sample_number', 'gene_number'] + name_list
    df = pd.read_csv(pan_matrix_path, sep='\t', names=['SG', 'gene'])
    df['gene_number'] = df['gene'].apply(lambda x: x.count('_'))
    df['sample_number'] = df['gene'].apply(lambda x: len(set(g.split('_')[0] for g in x.split(','))))

    raw_df = df.apply(process_genes, axis=1, gene_lengths=gene_len_dic)
    for name in name_list:
        raw_df[name] = raw_df['gene'].apply(lambda x: next((g.split('_')[1] for g in x.split(',') if g.startswith(name)), ''))

    polished_df = raw_df[order_list]
    return polished_df


def draw(polished_df,name_list):
    plt.figure(figsize=(8,12))
    sns.set(style="white")
    palette = sns.color_palette("viridis", len(name_list))
    index = [x for x in range(1, len(name_list) + 1)]
    data = pd.DataFrame(polished_df['sample_number'].value_counts()).reset_index()
    barplot = sns.barplot(x=data['sample_number'], y=data['count'], order=index, palette=palette)

    # 设置标题和标签
    barplot.set_xlabel('Shared_Genomes', fontsize=14)
    barplot.set_ylabel('SOG_Number', fontsize=14)

    return barplot

def main(config, pan_matrix,outpng,outsvg,outpdf, verbose):
    set_logging_level(verbose)
    gene_len_dic = {}
    name_list = []
    for line in open(config, 'rt').read().rstrip().split('\n'):
        sample, _, gff = line.split('\t')
        name_list.append(sample)
        gene_len_dic |= gene_length_from_gff(sample,gff)

    polished_df = polish_pan_matrix(pan_matrix,name_list,gene_len_dic)
    polished_df.to_csv(pan_matrix+'.pan', sep='\t', index=False)
    data = pd.DataFrame(polished_df['sample_number'].value_counts()).sort_values(by='sample_number').reset_index()
    plot = draw(polished_df,name_list)
    if outpng:
        outplot = pan_matrix + '.png'
        plot.get_figure().savefig(outplot, format='png',dpi=300)
    elif outsvg:
        outplot = pan_matrix + '.svg'
        plot.get_figure().savefig(outplot, format='png', dpi=300)
    elif outpdf:
        outplot = pan_matrix + '.pdf'
        plot.get_figure().savefig(outplot, format='png', dpi=300)
    fo = open(pan_matrix+'.stat','wt')
    logging.info('-----------------------------------')
    logging.info('Shared_Genomes\tSOG_Number')
    for index, row in data.iterrows():
        logging.info(f"{row['sample_number']}\t{row['count']}")
        fo.write(f"{row['sample_number']}\t{row['count']}\n")
    logging.info('-----------------------------------')
    logging.info(f'All\t{len(polished_df)}')

if __name__ == '__main__':
    main()



import logging

import gffutils
import pathlib

from pansg._logging import set_logging_level
from Bio import SeqIO, Seq, SeqRecord


# 创建或加载GFF数据库
def create_gff_db(gff, db):
    if pathlib.Path(db).is_file():
        return gffutils.FeatureDB(db, keep_order=True)
    else:
        logging.info("build your gff.db now")
        return gffutils.create_db(gff, dbfn=db, force=True, keep_order=True, merge_strategy='merge',
                                  sort_attribute_values=True)


# 获取转录本的序列
def get_transcript_sequence(transcript, genome_seq, gff_db):
    cds_seqs = []
    for cds in gff_db.children(transcript, featuretype='CDS', order_by='start'):
        cds_seq = genome_seq[cds.chrom].seq[cds.start - 1:cds.end]
        cds_seqs.append(cds_seq)
    transcript_seq = Seq.Seq('').join(cds_seqs)
    if transcript.strand == '-':
        transcript_seq = transcript_seq.reverse_complement()
    return transcript_seq


def main(config: str, verbose: str):
    set_logging_level(verbose)
    for line in open(config).read().rstrip().split('\n'):
        sample, fasta, gff = line.split('\t')
        nuc_seq = SeqIO.to_dict(SeqIO.parse(fasta, 'fasta'))
        gff_db = create_gff_db(gff, f'{pathlib.Path(gff).parent}/{pathlib.Path(gff).stem}.db')

        longest_transcripts = {}

        for gene in gff_db.features_of_type('gene'):
            transcripts = list(gff_db.children(gene, featuretype='mRNA'))
            if not transcripts:
                continue

            # 计算每个转录本的外显子长度之和，并找到最长的转录本
            longest_transcript = max(transcripts, key=lambda t: sum(
                exon.end - exon.start + 1 for exon in gff_db.children(t, featuretype='exon')))
            # 获取转录本的序列
            transcript_seq = get_transcript_sequence(longest_transcript, nuc_seq, gff_db)

            # 翻译为氨基酸序列
            pro_seq = transcript_seq.translate(to_stop=True, stop_symbol='')
            header = longest_transcript.id
            record = SeqRecord.SeqRecord(pro_seq, id=header, description='')
            longest_transcripts[header] = record

        SeqIO.write(longest_transcripts.values(), f"{pathlib.Path(fasta).parent}/{sample}.pro_fasta",
                    "fasta")


if __name__ == '__main__':
    main('data/pansg.config')



import logging
import re

import gffutils

from pansg._logging import set_logging_level
from pathlib import Path


def cluster_tandem(tandem_pwd):
    fi = open(tandem_pwd, 'rt')
    sample_name = tandem_pwd.split('/')[-3]
    fi.readline()
    tandem_dic = {}
    last_key = ""
    lines = fi.readline()
    while lines:
        line = lines.rstrip().split('\t')
        target = sample_name + "_" + re.split(r'\.|\_|\-|\:', line[0])[0]
        repeat = sample_name + "_" + re.split(r'\.|\_|\-|\:', line[2])[0]
        if tandem_dic.get(target) == 1:
            tandem_dic[last_key] = tandem_dic[last_key] + "," + repeat
        else:
            tandem_dic[target] = repeat
            last_key = target
        tandem_dic[repeat] = 1
        lines = fi.readline()

    return tandem_dic


def sg2syn_dic(right: str, left: str, synpair: dict) -> dict:
    insyn = open(f"{right}_{left}/{right}_{left}.dag.aligncoords", 'rt')
    for lines in insyn:
        # 共线性块信息没有用
        if lines.startswith('#'):
            pass
        else:
            _, QG, _, _, _, RG, _, _, _, _ = lines.rstrip().split('\t')

            QG = right + "_" + re.split(r'\.|\_|\-|\:', QG)[0]
            RG = left + "_" + re.split(r'\.|\_|\-|\:', RG)[0]
            if len(re.split(r'\.|\_|\-|\:', QG)) > 2:
                logging.info("Only allow ., _, -, or : as separators between gene IDs and transcripts.")
            # 只保留最佳共线性对
            if QG in synpair.keys() or RG in synpair.keys():
                pass
            else:
                synpair[QG] = RG
                synpair[RG] = QG
    insyn.close()

    return synpair


def merge_gene2sg(QG: str, sg: dict, sg_rev: dict, synpair: dict) -> (dict, dict):
    sg_gene = synpair[QG]
    del sg[sg_rev[QG]]
    sg_id = sg_rev[sg_gene]
    sg[sg_id] = f"{sg[sg_id]},{QG}"
    sg_rev[QG] = sg_id
    return sg, sg_rev


def main(config, output, verbose):
    set_logging_level(verbose)
    fo = open(output, 'wt')

    td_pwd_list = []
    sample_list = []
    gff_dic = {}
    all_td_dic = {}

    sg = {}
    sg_rev = {}
    sg_order = 0
    scaffold_keywords = ["scaf", "scaffold", "contig", "SK", "unplaced", "UN"]
    for line in open(config, 'rt').read().rstrip().split('\n'):
        sample, _, gff = line.split('\t')
        td_pwd_list.append(f"{sample}/result/{sample}.tandem.pairs")
        sample_list.append(sample)
        gff_dic[sample] = gff
    # 所有的串联重复关系，将value 还原为 key
    for pwd in td_pwd_list:
        all_td_dic |= cluster_tandem(pwd)

    # 读取每个材料的所有基因，提供初始编号
    for sample, gff in gff_dic.items():
        db = gffutils.FeatureDB(str(Path(gff).with_suffix('.db')))
        for feature in db.features_of_type('gene'):
            if not any(keyword in feature.chrom for keyword in scaffold_keywords):
                gene = sample + "_" + feature.id
                if all_td_dic.get(gene) == 1:
                    # 去除串联重复基因
                    pass
                else:
                    # 初始化
                    sg[sg_order] = gene
                    sg_rev[gene] = sg_order
                    sg_order += 1

    for i in range(1, len(sample_list)):
        right = sample_list[i]
        unsyn = []
        for j in range(0, i):
            left = sample_list[j]
            logging.info(f"Current comparison: {right} vs {left}... \n")
            # 读取Mo17 和 B73的所有共线性对
            # 处理Oh7B的时候会先读Mo17后读B73
            synpair = sg2syn_dic(right, left, {})
            synpair = sg2syn_dic(left, right, synpair)

            # 读取Mo17的所有基因
            if j == 0:
                Rgff = gffutils.FeatureDB(str(Path(gff_dic[right]).with_suffix('.db')))
                for feature in Rgff.features_of_type('gene'):
                    if not any(keyword in feature.chrom for keyword in scaffold_keywords):
                        QG = right + "_" + feature.id
                        if not synpair.get(QG):
                            # 和Mo17不共线的等待后续处理
                            unsyn.append(QG)
                        else:
                            sg, sg_rev = merge_gene2sg(QG, sg, sg_rev, synpair)

                logging.info(f"Non-syntenic-to-Ref Gene Number:{len(unsyn)}!!!\n")
                logging.info("Compare to Non-Ref genomes...\n===============\n")

            else:
                # 现在处理 B73 和 Mo17 不共线的基因
                # 当i=2的时候则是处理 Oh7B 和 Mo17 不共线的基因
                unsyn_tmp = []
                logging.info(f"**Current Non-Ref Comparison: {right} vs {left}... \n==============\n")
                for private_gene in unsyn:
                    if synpair.get(private_gene):

                        if j == 1:
                            sg, sg_rev = merge_gene2sg(private_gene, sg, sg_rev, synpair)
                        else:
                            private_id = sg_rev[private_gene]
                            mediate_sg_id = sg_rev[synpair[private_gene]]
                            if private_id == mediate_sg_id:
                                pass
                            else:
                                if sg[private_id] == private_gene:
                                    sg, sg_rev = merge_gene2sg(private_gene, sg, sg_rev, synpair)
                                else:
                                    sg[private_id] = f"{sg[private_id]},{sg[mediate_sg_id]}"
                                    for gene in sg[mediate_sg_id].split(","):
                                        sg_rev[gene] = private_id
                                    del sg[mediate_sg_id]
                    else:
                        unsyn_tmp.append(private_gene)
                unsyn = unsyn_tmp

    for i, (_, v) in enumerate(sorted(sg.items(), key=lambda kv: (kv[1], kv[0]))):
        for gene in v.split(","):
            if all_td_dic.get(gene, 1) != 1:
                # 解压串联重复基因
                v = v.replace(gene, gene + "," + all_td_dic[gene])
        idx = f"SG{i + 1:07d}"
        fo.write(f'{idx}\t{v}\n')
    fo.close()


if __name__ == '__main__':
    main("data/pansg.config",
         'SG_test', 'INFO')



import logging
import subprocess
import gffutils
import shutil

from pansg._pool import parallel_process
from pansg._logging import set_logging_level
from pathlib import Path

def install_software(software, command):
    """安装软件包"""
    if shutil.which(command):
        pass
    else:
        if command == 'DupGen_finder.pl':
            logging.warning("please install DupGen_finder by yourself")
            logging.warning('git clone https://github.com/qiao-xin/DupGen_finder.git')
            logging.warning('cd DupGen_finder')
            logging.warning(
                f'make\nchmod 775 DupGen_finder.pl\nchmod 775 DupGen_finder-unique.pl\nchmod 775 set_PATH.sh\nsource set_PATH.sh')
            raise
        else:
            try:
                logging.info(f"{software} not found. Installing {software} using Conda...")
                subprocess.run(["conda", "install", "-c", "bioconda", "-y", software], check=True)
                logging.info(f"{software} installation completed.")
            except subprocess.CalledProcessError as e:
                logging.info(f"An error occurred during {software} installation: {e}")

def run_blastp(query, sub, out, thread, evalue):
    # 运行blastp
    if Path(out).is_file():
        pass
    else:
        subprocess.run(['blastp',
                        '-evalue', str(evalue),
                        '-query', query,
                        '-db', sub,
                        '-out', out,
                        '-outfmt', '6',
                        '-num_threads', str(thread)], stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)


def DupGen_finder(sample: str, gff: Path):
    # 处理gff文件
    db = gffutils.FeatureDB(str(gff.with_suffix('.db')))
    fo = open(f"{sample}/{sample}.gff", 'wt')
    for mrna in db.features_of_type('mRNA'):
        fo.write(f"{mrna.chrom}\t{mrna.id}\t{mrna.start}\t{mrna.end}\n")
    try:
        Path(f"{sample}/{sample}.blast").symlink_to(Path(f"./{sample}/{sample}_{sample}.blast").resolve())
        Path(f"{sample}/{sample}_{sample}.gff").symlink_to(Path(f"{sample}/{sample}.gff").resolve())
    except FileExistsError:
        logging.info(f"{sample}/{sample}.gff or {sample}/{sample}.blast is existed")
    subprocess.run(['DupGen_finder.pl',
                    '-i', sample,
                    '-t', sample,
                    '-c', sample,
                    '-o', f"{sample}/result"], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)


def cluster_tandem(tandem_pwd):
    fi = open(tandem_pwd, 'rt')
    fi.readline()
    tandem_dic = {}
    last_key = ""
    lines = fi.readline()
    while lines:
        line = lines.rstrip().split('\t')
        if tandem_dic.get(line[0]) == 1:
            tandem_dic[last_key] = tandem_dic[last_key] + "," + line[2]
        else:
            tandem_dic[line[0]] = line[2]
            last_key = line[0]
        tandem_dic[line[2]] = 1
        lines = fi.readline()

    return tandem_dic


def run_dag_chainer(blastp_out, D, g, A):
    # 将blastp输出结果转化为dagchainer的输入，并最终调用
    ref, que = blastp_out.split('/')[-2].split('_')
    dag_fn = str(Path(blastp_out).with_suffix('.dag').resolve())
    temp_fo = open(dag_fn, 'wt')
    r_gff = open(f"{ref}/{ref}.gff", 'rt')
    q_gff = open(f"{que}/{que}.gff", 'rt')

    r_td_dic = cluster_tandem(f"{ref}/result/{ref}.tandem.pairs")
    q_td_dic = cluster_tandem(f"{que}/result/{que}.tandem.pairs")
    qs = {}
    qe = {}
    qchr = {}
    rs = {}
    re = {}
    rchr = {}

    lines = r_gff.readline()
    while lines:
        r_chr, gene, r_s, r_e = lines.rstrip().split("\t")
        rs[gene] = r_s
        re[gene] = r_e
        rchr[gene] = r_chr
        lines = r_gff.readline()
    lines = q_gff.readline()
    r_gff.close()
    while lines:
        q_chr, gene, q_s, q_e = lines.rstrip().split("\t")
        qs[gene] = q_s
        qe[gene] = q_e
        qchr[gene] = q_chr
        lines = q_gff.readline()
    q_gff.close()

    fi = open(blastp_out, 'rt')
    lines = fi.readline()
    while lines:
        line = lines.rstrip().split('\t')
        qblast = line[1]
        rblast = line[0]
        pvalue = line[-2]
        if qblast not in qs.keys() or rblast not in rs.keys():
            # 去掉不存在于gff文件中的基因
            break
        else:
            if rchr[rblast] != qchr[qblast]:
                # 去掉不同染色体上的比对
                pass
            else:
                if q_td_dic.get(qblast) or r_td_dic.get(rblast):
                    # 去掉串联重复(将在后续步骤中返回流程)
                    pass
                else:
                    temp_fo.write(
                        f"{ref}_{rchr[rblast]}\t{rblast}\t{rs[rblast]}\t{re[rblast]}\t{que}_{qchr[qblast]}\t{qblast}\t{qs[qblast]}\t{qe[qblast]}\t{pvalue}\n")

        lines = fi.readline()
    # 运行 DAG chainer
    try:
        subprocess.run(['run_DAG_chainer.pl', '-s', '-I',
                        '-i', dag_fn,
                        '-D', str(D),
                        '-g', str(g),
                        '-A', str(A)], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
    except subprocess.CalledProcessError as e:
        logging.error(f"An error occurred while running run_DAG_chainer.pl: {e.stderr}")
        raise


def process_pairs(pair, thread, evalue, D, g, A):
    tasks = []
    for x, y in pair:
        Path(f'{Path(x).stem}_{Path(y).stem}').mkdir(parents=True, exist_ok=True)
        blastp_out = f'{Path(x).stem}_{Path(y).stem}/{Path(x).stem}_{Path(y).stem}.blast'
        logging.info(blastp_out + ' run blastp now')
        Path(f"{Path(x).stem}_{Path(y).stem}").mkdir(parents=True, exist_ok=True)
        run_blastp(x, y, blastp_out, thread, evalue)
        tasks.append((run_dag_chainer, blastp_out, D, g, A))
    return tasks


def main(config: str, thread: int = 8, D: int = 200000, g: int = 10000,
         A: int = 6, evalue: float = 1e-20, verbose='ERROR'):
    set_logging_level(verbose)
    pro_fasta_list = []
    gff_dic = {}
    for line in open(config, 'rt').read().rstrip().split('\n'):
        sample, fasta, gff = line.split('\t')
        Path(sample).mkdir(parents=True, exist_ok=True)
        pro_fasta_list.append(f"{Path(fasta).parent}/{sample}.pro_fasta")
        gff_dic[sample] = Path(gff)
    pair = []
    install_software('blast', 'makeblastdb')


    for i in range(0, len(pro_fasta_list)):
        subprocess.run(['makeblastdb',
                        '-in', pro_fasta_list[i],
                        '-dbtype', 'prot',
                        '-out', pro_fasta_list[i]], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                       text=True)
        # 鉴定串联重复
        sample_dir = Path(pro_fasta_list[i]).stem
        run_blastp(pro_fasta_list[i], pro_fasta_list[i],
                   f'{sample_dir}/{Path(pro_fasta_list[i]).stem}_{sample_dir}.blast', thread, evalue)
        install_software('DupGen_finder', 'DupGen_finder.pl')
        DupGen_finder(sample_dir, gff_dic[sample_dir])
        for j in range(i + 1, len(pro_fasta_list)):
            pair.append([pro_fasta_list[i], pro_fasta_list[j]])
            pair.append([pro_fasta_list[j], pro_fasta_list[i]])
    install_software("dagchainer", "run_DAG_chainer.pl")
    tasks = process_pairs(pair, thread, evalue, D, g, A)
    try:
        parallel_process(tasks, num_processes=min(len(tasks), thread, 1))  # 并行处理 run_dag_chainer 任务
    except TypeError:
        logging.info("Why NoneType?")
        raise


if __name__ == '__main__':
    D = 200000
    g = 10000
    A = 6
    run_dag_chainer('Mo17_B73/B73_Mo17.blast', D, g, A)



import subprocess
import shutil
import logging

from pansg._logging import set_logging_level
def install_software(software, command):
    """安装软件包"""
    if shutil.which(command):
        pass
    else:
        if command == 'DupGen_finder.pl':
            logging.warning("please install DupGen_finder by yourself")
            logging.warning('git clone https://github.com/qiao-xin/DupGen_finder.git')
            logging.warning('cd DupGen_finder')
            logging.warning(
                f'make\nchmod 775 DupGen_finder.pl\nchmod 775 DupGen_finder-unique.pl\nchmod 775 set_PATH.sh\nsource set_PATH.sh')
            raise
        else:
            try:
                logging.info(f"{software} not found. Installing {software} using Conda...")
                subprocess.run(["conda", "install", "-c", "bioconda", "-y", software], check=True)
                logging.info(f"{software} installation completed.")
            except subprocess.CalledProcessError as e:
                logging.info(f"An error occurred during {software} installation: {e}")


def main(config, pan_matrix, verbose):
    set_logging_level(verbose)
    install_software('miniprot', 'miniprot')
    install_software('bedtools', 'bedtools')
    install_software('parallel','parallel')
    subprocess.run(['bash',
                    'run_zzq.sh',
                    config, pan_matrix], shell=True, check=True)



# pool.py
import logging
from multiprocessing import Pool

def parallel_process(tasks, num_processes):
    with Pool(processes=num_processes) as pool:
        pool.starmap(process_task, tasks)


def process_task(func, *args):
    try:
        return func(*args)
    except Exception as e:
        logging.error(f"Task failed with exception: {e}")
        return f"Task failed with exception: {e}"


if __name__ == "__main__":
    # 示例代码，可以在运行 pool.py 时测试
    sample_tasks = [1, 2, 3, 4, 5]  # 示例任务列表
    parallel_process(sample_tasks)



import logging
from logging import (CRITICAL, ERROR, WARNING, INFO, DEBUG, NOTSET)


def set_logging_level(level='ERROR'):
    level = level.upper()
    dt_level = {
        k: v for k, v in zip(
            ['CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG', 'NOTSET'],
            [CRITICAL, ERROR, WARNING, INFO, DEBUG, NOTSET]
        )
    }
    logging.basicConfig(level=dt_level[level])

    logging.info(f'set logging level to {level}')


if __name__ == '__main__':
    set_logging_level('NOTSET')

    logging.debug('Python debug')
    logging.info('Python info')
    logging.warning('Python warning')
    logging.error('Python error')
    logging.critical('Python critical')



from fire import Fire
from pansg.blastp_2_dagchainer import main as blastp_2_dagchainer
from pansg.extract_longest_protein import main as extract_longest_protein
from pansg.merge import main as merge
from pansg.polish import main as polish
from pansg.zzq import main as zzq


class Cli(object):
    """make your syntelog-based pan-genome

    - pansa is a commandline tool and a python package to build a syntelog-based pan-genome matrix.
    - It is recommended to create a new folder to run the processes in this software.
    - To avoid unnecessary bugs, it is recommended to retain only the information from the chromosomal regions and remove the scaffold regions in the input GFF file.
    """

    def extract_longest_protein(self, config: str, verbose: str = 'ERROR'):
        """ Extract the longest amino acid sequence for each gene from the DNA sequence file using the GFF3 annotation file for subsequent synteny identification.
        :param config: <sample_name>    <fasta_path>    <gff3_path>
        :param verbose: CRITICAL, ERROR, WARNING, INFO, DEBUG, NOTSET
        :return :
        """
        return extract_longest_protein(config=config, verbose=verbose)

    def blastp_2_dagchainer(self, config: str, thread: int = 8, D: int = 1000000, g: int = 40000,
                            A: int = 5, evalue: float = 1e-5, verbose: str = 'ERROR'):
        """ Run dagchainer
        :param config: <sample_name>    <fasta_path>    <gff3_path>
        :param thread: thread number
        :param D: maximum distance allowed between two matches in basepairs
        :param g: length of a gap in bp
        :param A: Minium number of Aligned Pairs
        :param evalue: E-value threshold for blastp. A stricter parameter results in a faster DAGchainer run. It is an important parameter
        :param verbose: CRITICAL, ERROR, WARNING, INFO, DEBUG, NOTSET
        :return:
        """
        return blastp_2_dagchainer(config=config, thread=thread, D=D, g=g, A=A, evalue=evalue, verbose=verbose)

    def merge(self, config: str, output: str = 'SG_test', verbose: str = 'ERROR'):
        """ Merge the results of syntelog
        :param config: <sample_name>    <fasta_path>    <gff3_path>  You can merge only the part of the samples you want.\
        The first sample will serve as the reference genome for subsequent merged syntelog matrices. Please select the assembly with the highest quality.
        :param output: your raw syntelog-based pan-genome matrix
        :param verbose: CRITICAL, ERROR, WARNING, INFO, DEBUG, NOTSET
        :return:
        """
        return merge(config=config, output=output, verbose=verbose)

    def polish(self, config: str, pan_matrix: str = 'SG_test', outpng: bool = True, outsvg: bool = False,
               outpdf: bool = False, verbose: str = 'INFO'):
        """
        :param config: <sample_name>    <fasta_path>    <gff3_path>
        :param pan_matrix: your syntelog-based pan-genome matrix pathway
        :param outpng: whether to save the pan-genome matrix in png format
        :param outsvg: whether to save the pan-genome matrix in svg format
        :param outpdf: whether to save the pan-genome matrix in pdf format
        :param verbose: CRITICAL, ERROR, WARNING, INFO, DEBUG, NOTSET
        :return:
        """
        return polish(config=config, pan_matrix=pan_matrix, outpng=outpng, outsvg=outsvg, outpdf=outpdf,
                      verbose=verbose)

    def zzq(self, config: str, pan_matrix: str = 'SG_test.pan', verbose: str = 'INFO'):
        """

        :param config: <sample_name>    <fasta_path>    <gff3_path>
        :param pan_matrix: your syntelog-based pan-genome matrix after polishing
        :param verbose: CRITICAL, ERROR, WARNING, INFO, DEBUG, NOTSET
        :return:
        """
        return zzq(config=config, pan_matrix=pan_matrix, verbose=verbose)


def main():
    cli = Cli()
    Fire(cli, name='pansg')


if __name__ == '__main__':
    main()
